@@ -1,11 +1,14 @@
 from collections import namedtuple
 
 import kubernetes
+import yaml
 
-from dagster import Array, Field, Noneable
+from dagster import Array, DagsterInvariantViolationError, Field, Noneable, StringSource
 from dagster import __version__ as dagster_version
 from dagster import check
+from dagster.config.field_utils import Shape
 from dagster.serdes import whitelist_for_serdes
+from dagster.utils import frozentags, merge_dicts
 
 K8S_JOB_BACKOFF_LIMIT = 4
 
@@ -26,6 +29,32 @@ DAGSTER_PG_PASSWORD_SECRET_KEY = 'postgresql-password'
 # Kubernetes Job object names cannot be longer than 63 characters
 MAX_K8S_NAME_LEN = 63
 
+K8S_RESOURCE_REQUIREMENTS_KEY = 'dagster-k8s/resource_requirements'
+K8s_RESOURCE_REQUIREMENTS_VALID_KEYS = set(['limits', 'requests'])
+
+
+def get_k8s_resource_requirements(tags):
+    check.inst_param(tags, 'tags', frozentags)
+    req_str = tags.get(K8S_RESOURCE_REQUIREMENTS_KEY)
+    if req_str is not None:
+        req_dict = yaml.safe_load(req_str)
+
+        req_keys = set(req_dict.keys())
+        if len(req_keys.difference(K8s_RESOURCE_REQUIREMENTS_VALID_KEYS)) > 0:
+            raise DagsterInvariantViolationError(
+                'Invalid K8s resource specification. {resource} expected to only contain keys in '
+                'set {valid_keys} but found extra keys {extra_keys}'.format(
+                    resource=req_dict,
+                    valid_keys=K8s_RESOURCE_REQUIREMENTS_VALID_KEYS,
+                    extra_keys=req_keys.difference(K8s_RESOURCE_REQUIREMENTS_VALID_KEYS),
+                )
+            )
+
+        req = kubernetes.client.V1ResourceRequirements(**req_dict)
+        return req
+
+    return None
+
 
 @whitelist_for_serdes
 class DagsterK8sJobConfig(
@@ -103,64 +132,81 @@ class DagsterK8sJobConfig(
 
     @classmethod
     def config_type(cls):
+        '''Combined config type which includes both run launcher and pipeline run config.
+        '''
+        cfg_run_launcher = DagsterK8sJobConfig.config_type_run_launcher()
+        cfg_pipeline_run = DagsterK8sJobConfig.config_type_pipeline_run()
+        return merge_dicts(cfg_run_launcher, cfg_pipeline_run)
+
+    @classmethod
+    def config_type_run_launcher(cls):
+        '''Configuration intended to be set on the Dagster instance.
+        '''
         return {
-            'job_image': Field(
-                str,
-                is_required=True,
-                description='Docker image to use for launched task Jobs '
-                '(e.g. "mycompany.com/dagster-k8s-image:latest").',
-            ),
-            'dagster_home': Field(
-                str,
-                is_required=False,
-                default_value=DAGSTER_HOME_DEFAULT,
-                description='The location of DAGSTER_HOME in the Job container; this is where the '
-                '``dagster.yaml`` file will be mounted from the instance ConfigMap specified here. '
-                'Defaults to /opt/dagster/dagster_home.',
-            ),
-            'image_pull_policy': Field(
-                str,
-                is_required=False,
-                default_value='IfNotPresent',
-                description='Image pull policy to set on the launched task Job Pods. Defaults to '
-                '"IfNotPresent".',
-            ),
-            'image_pull_secrets': Field(
-                Array(str),
-                is_required=False,
-                description='(Advanced) Specifies that Kubernetes should get the credentials from '
-                'the Secrets named in this list.',
-            ),
-            'service_account_name': Field(
-                Noneable(str),
-                is_required=False,
-                description='(Advanced) Override the name of the Kubernetes service account under '
-                'which to run the Job.',
-            ),
             'instance_config_map': Field(
-                str,
+                StringSource,
                 is_required=True,
                 description='The ``name`` of an existing Volume to mount into the pod in order to '
                 'provide a ConfigMap for the Dagster instance. This Volume should contain a '
                 '``dagster.yaml`` with appropriate values for run storage, event log storage, etc.',
             ),
             'postgres_password_secret': Field(
-                str,
+                StringSource,
                 is_required=True,
                 description='The name of the Kubernetes Secret where the postgres password can be '
                 'retrieved. Will be mounted and supplied as an environment variable to the Job Pod.'
                 'Secret must contain the key ``"postgresql-password"`` which will be exposed in '
                 'the Job environment as the environment variable ``DAGSTER_PG_PASSWORD``.',
             ),
+            'dagster_home': Field(
+                StringSource,
+                is_required=False,
+                default_value=DAGSTER_HOME_DEFAULT,
+                description='The location of DAGSTER_HOME in the Job container; this is where the '
+                '``dagster.yaml`` file will be mounted from the instance ConfigMap specified here. '
+                'Defaults to /opt/dagster/dagster_home.',
+            ),
+        }
+
+    @classmethod
+    def config_type_pipeline_run(cls):
+        '''Configuration intended to be set at pipeline execution time.
+        '''
+        return {
+            'job_image': Field(
+                StringSource,
+                is_required=True,
+                description='Docker image to use for launched task Jobs '
+                '(e.g. "mycompany.com/dagster-k8s-image:latest").',
+            ),
+            'image_pull_policy': Field(
+                StringSource,
+                is_required=False,
+                default_value='IfNotPresent',
+                description='Image pull policy to set on the launched task Job Pods. Defaults to '
+                '"IfNotPresent".',
+            ),
+            'image_pull_secrets': Field(
+                Array(Shape({'name': StringSource})),
+                is_required=False,
+                description='(Advanced) Specifies that Kubernetes should get the credentials from '
+                'the Secrets named in this list.',
+            ),
+            'service_account_name': Field(
+                Noneable(StringSource),
+                is_required=False,
+                description='(Advanced) Override the name of the Kubernetes service account under '
+                'which to run the Job.',
+            ),
             'env_config_maps': Field(
-                Noneable(list),
+                Noneable(Array(StringSource)),
                 is_required=False,
                 description='A list of custom ConfigMapEnvSource names from which to draw '
                 'environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:'
                 'https://kubernetes.io/docs/tasks/inject-data-application/define-environment-variable-container/#define-an-environment-variable-for-a-container',
             ),
             'env_secrets': Field(
-                Noneable(list),
+                Noneable(Array(StringSource)),
                 is_required=False,
                 description='A list of custom Secret names from which to draw environment '
                 'variables (using ``envFrom``) for the Job. Default: ``[]``. See:'
@@ -199,7 +245,9 @@ class DagsterK8sJobConfig(
         return DagsterK8sJobConfig(**config)
 
 
-def construct_dagster_graphql_k8s_job(job_config, args, job_name, pod_name=None, component=None):
+def construct_dagster_graphql_k8s_job(
+    job_config, args, job_name, resources=None, pod_name=None, component=None
+):
     '''Constructs a Kubernetes Job object for a dagster-graphql invocation.
 
     Args:
@@ -207,6 +255,8 @@ def construct_dagster_graphql_k8s_job(job_config, args, job_name, pod_name=None,
             Job object.
         args (List[str]): CLI arguments to use with dagster-graphql in this Job.
         job_name (str): The name of the Job. Note that this name must be <= 63 characters in length.
+        resources (kubernetes.client.V1ResourceRequirements): The resource requirements for the
+            container
         pod_name (str, optional): The name of the Pod. Note that this name must be <= 63 characters
             in length. Defaults to "<job_name>-pod".
         component (str, optional): The name of the component, used to provide the Job label
@@ -218,6 +268,9 @@ def construct_dagster_graphql_k8s_job(job_config, args, job_name, pod_name=None,
     check.inst_param(job_config, 'job_config', DagsterK8sJobConfig)
     check.list_param(args, 'args', of_type=str)
     check.str_param(job_name, 'job_name')
+    resources = check.opt_inst_param(
+        resources, 'resources', kubernetes.client.V1ResourceRequirements
+    )
     pod_name = check.opt_str_param(pod_name, 'pod_name', default=job_name + '-pod')
     check.opt_str_param(component, 'component')
 
@@ -262,6 +315,7 @@ def construct_dagster_graphql_k8s_job(job_config, args, job_name, pod_name=None,
             ),
         ],
         env_from=job_config.env_from_sources,
+        resources=resources,
         volume_mounts=[
             kubernetes.client.V1VolumeMount(
                 name='dagster-instance',
@@ -281,7 +335,10 @@ def construct_dagster_graphql_k8s_job(job_config, args, job_name, pod_name=None,
     template = kubernetes.client.V1PodTemplateSpec(
         metadata=kubernetes.client.V1ObjectMeta(name=pod_name, labels=dagster_labels),
         spec=kubernetes.client.V1PodSpec(
-            image_pull_secrets=job_config.image_pull_secrets,
+            image_pull_secrets=[
+                kubernetes.client.V1LocalObjectReference(name=x['name'])
+                for x in job_config.image_pull_secrets
+            ],
             service_account_name=job_config.service_account_name,
             restart_policy='Never',
             containers=[job_container],
